/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"
#include "PartIdList.h"


//////////////////////////////////////////////////////////////////
//
// Reorganization Test #1 - Visit all composite parts, deleting
// and then newly inserting 1/2 of each composite part's atomic
// parts graph.
//
//////////////////////////////////////////////////////////////////


extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Reorg1
//
//////////////////////////////////////////////////////////////////


int reorg1()
{
}
/*
// iterate through all the composite parts, deleting/inserting
    // half of an atomic parts graph worth of parts and counting up
    // the number of atomic parts processed in this way

    int count = 0;
    foreach (CompositePart* comp, *CompositePart::extent) {
        if (debugMode) {
            printf("    In Reorg1, partId = %d:\n", comp->id);
        }
        count += comp->reorg1();
    }

    return count;
}


//////////////////////////////////////////////////////////////////
//
// Composite Part Methods for Reorg1
//
//////////////////////////////////////////////////////////////////


int CompositePart::reorg1()
{
    if (debugMode) {
        printf("CompositePart::Reorg1(id = %d)\n", id);
    }

    // start by getting pointers to all of the atomic parts, in increasing
    // part id order, in preparation for removing/recreating half of them

    int partId, i;
    AtomicPart** atom = new AtomicPart* [NumAtomicPerComp];
    int lowId  = rootPart->id;
    int highId = lowId + NumAtomicPerComp - 1;

    partId = lowId;
    for (i = 0; i < NumAtomicPerComp; i++) {
	atom[i] = (*AtomicPart::extent)[% id == partId %];
	partId++;
    }

    // now do the deletions of half of the atomic parts, keeping the first
    // half and targeting the second half for simplicity (to avoid the root
    // part and to enable a simple computation to discriminate between parts
    // to be deleted and those to be retained);
    // before doing the deletions, go through and remember all the connections
    // that will be lost when the targeted parts are deleted;
    // after all that, recreate the parts and reconnect them to everything!

    // first compute the range of impact

    int amtToChange, firstToChange, midId;
    amtToChange = NumAtomicPerComp / 2;
    firstToChange = NumAtomicPerComp - amtToChange;
    midId = highId - amtToChange + 1;

    // next record the relevant connections

    PartIdList* toParts = new PartIdList [NumAtomicPerComp];
    for (i = 0; i < NumAtomicPerComp; i++) {
        Connection* conn;
        foreach (conn, atom[i]->to) {
	    if ((atom[i]->id >= midId) || (conn->to->id >= midId)) {
		toParts[i].insert(conn->to->id);
            }
        }
    }

    // now do the actual part deletions

    int count = 0;
    for (i = firstToChange; i < NumAtomicPerComp; i++) {
	if (debugMode) {
            printf("\tReorg1: Deleting atomic part %d\n", atom[i]->id);
	}
	delete atom[i];
	count++;
    }

    // time to (approximately) recreate the deleted parts

    partId = midId;
    for (i = firstToChange; i < NumAtomicPerComp; i++) {
	atom[i] = new (os_segment::of(this)) AtomicPart (partId);
	parts |= atom[i];
	if (debugMode) {
            printf("\tReorg1: Recreated atomic part %d\n", atom[i]->id);
	}
	partId++;
    }

    // finally, connect all of the parts back up again

    Connection* conn;
    for (i = 0; i < NumAtomicPerComp; i++) {
	while (!toParts[i].empty()) {
	    int to = toParts[i].remove() - lowId;
	    conn = new (os_segment::of(this)) Connection (atom[i], atom[to]);
	}
    }

    // give back temporary storage and then return final count
    delete [NumAtomicPerComp] atom;
    delete [NumAtomicPerComp] toParts;
    return count;
}

*/

//////////////////////////////////////////////////////////////////
//
// Reorganization Test #2 - Randomly delete and reinsert as many
// atomic parts per composite part as each one has, interleaving
// reorganizations of all of the composite parts, to test out the
// deletion/reallocation handling and performance characteristics
// of the system
//
//////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////
//
// Routine to do Reorg2
//
//////////////////////////////////////////////////////////////////


int reorg2()
{
}
/*
    // iterate through all of the composite parts, deleting and then
    // recreating one of the atomic parts from its atomic parts graph;
    // repeat this process NumAtomicPerComp times in order to cause major
    // changes per composite part overall;
    // count the number of atomic parts deleted/recreated in this way;

    int count = 0;
    CompositePart* comp;
    for (int i = 0; i < NumAtomicPerComp; i++) {
        foreach (comp, *CompositePart::extent) {
            if (debugMode) {
                printf("    In Reorg2, partId = %d:\n", comp->id);
            }
            count += comp->reorg2();
        }
    }

    return count;
}



//////////////////////////////////////////////////////////////////
//
// Composite Part Methods for Reorg2
//
//////////////////////////////////////////////////////////////////


int CompositePart::reorg2()
{
    if (debugMode) {
        printf("CompositePart::Reorg2(id = %d)\n", id);
    }

    // first select and locate a victim atomic part

    int lowId = rootPart->id;
    int victimId = lowId + (int) (random() % NumAtomicPerComp);
    AtomicPart* victim = (*AtomicPart::extent)[% id == victimId %];

    // next record the affected to and from connections

    PartIdList toList;
    PartIdList fromList;
    Connection* conn;

    foreach (conn, victim->to) {
	toList.insert(conn->to->id);
    }
    foreach (conn, victim->from) {
	if (conn->from->id != victim->id) {
	    fromList.insert(conn->from->id);
        }
    }

    // now do the actual part deletion

    if (debugMode) {
        printf("\tReorg2: Deleting atomic part %d\n", victim->id);
    }
    delete victim;

    // time to (approximately) recreate the deleted part

    victim = new (os_segment::of(this)) AtomicPart (victimId);
    parts |= victim;
    if (victimId == lowId) {
       rootPart = victim;
    }
    if (debugMode) {
        printf("\tReorg2: Recreated atomic part %d\n", victim->id);
    }

    // finally, connect the part back up to the other parts again

    int partId;
    AtomicPart* toPart;
    AtomicPart* fromPart;

    while (!toList.empty()) {
	partId = toList.remove();
        toPart = (*AtomicPart::extent)[% id == partId %];
	conn = new (os_segment::of(this)) Connection (victim, toPart);
    }

    while (!fromList.empty()) {
	partId = fromList.remove();
        fromPart = (*AtomicPart::extent)[% id == partId %];
	conn = new (os_segment::of(this)) Connection (fromPart, victim);
    }

    return 1;
}

*/
